
from pathlib import Path

import git


def run(description: str, repo_path: Path) -> None:
    """
    Run the client: Create a commit on the coldbrew branch and push that branch to the coldbrew remote
    :param description: The commit message
    :param repo_path: Path to working directory containing .git/
    :return:
    """
    repo = git.Repo(repo_path)

    # Remember untracked files
    untracked_files = repo.untracked_files

    # Remember current head (usually just master)
    current_head = repo.head.reference

    # Switch to coldbrew branch (create it first if not present)
    coldbrew_head = repo.heads['coldbrew']
    repo.head.reference = coldbrew_head

    # Stage and commit all files
    repo.git.add(all=True)
    coldbrew_commit = repo.index.commit(description)
    print(f"-> Committed '{coldbrew_commit.message}' ({str(coldbrew_commit.hexsha)})")

    # Unstage previously untracked files and reset head
    repo.head.reference = current_head
    repo.head.reset(paths=untracked_files)

    # Push coldbrew branch first, then all additional ones
    coldbrew_remote = repo.remotes['coldbrew']
    coldbrew_remote.push(refspec=f"refs/heads/coldbrew:refs/heads/coldbrew")

    print(f"-> Pushed to {next(coldbrew_remote.urls)}")
    print("Done.")


def contains_git_repo(repo_path: Path) -> bool:
    """
    Check whether given directory contains a valid git repository
    :param repo_path: Path to directory
    :return:
    """
    try:
        git.Repo(repo_path)
    except git.InvalidGitRepositoryError:
        return False
    return True


def remote_exists(repo_path: Path) -> bool:
    """
    Check whether the repo in repo_path has a 'coldbrew' remote configured
    :param repo_path: Path to directory containing a .git/ directory
    :return:
    """
    repo = git.Repo(repo_path)
    return 'coldbrew' in repo.remotes


def remote_valid(repo_path: Path) -> bool:
    """
    Check whether the coldbrew remote in the repo in repo_path points to a reachable and valid git directory
    :param repo_path: Path to directory containing a .git/ directory
    :return:
    """
    repo = git.Repo(repo_path)
    try:
        repo.git.ls_remote('coldbrew')
    except git.GitCommandError as e:
        return False
    return True


def set_remote_url(repo_path: Path, url: str) -> None:
    """
    Set the remote url for the coldbrew remote in the given git repository
    :param repo_path: The path to the git repository
    :param url: The url to be set for the coldbrew remote
    :return:
    """
    repo = git.Repo(repo_path)
    remote = repo.remote('coldbrew')
    remote.set_url(url, next(remote.urls))


def init_coldbrew(repo_path: Path, url: str) -> None:
    """
    Initialize coldbrew (create coldbrew remote with url and coldbrew branch).
    Preconditions: 1. At least one commit in the repo and 2. No coldbrew branch yet
    :param repo_path: The path to the git repository
    :param url: The url to be set for the coldbrew remote
    :return:
    """
    import sys
    repo = git.Repo(repo_path)

    if len(repo.branches) == 0:
        print("Repository does not contain any branches. Make an initial commit to create a branch which coldbrew can switch back to after committing and pushing")
        print("Aborted.")
        import sys
        sys.exit(1)

    if 'coldbrew' in repo.branches:
        print("coldbrew branch exists already. If you are using it for something else, consider renaming it. "
              "If you initialized coldbrew before in this repo, make sure a remote called 'coldbrew' exists.")
        print("Aborted.")
        sys.exit(1)

    repo.create_remote('coldbrew', url)
    repo.create_head('coldbrew')


if __name__ == '__main__':
    raise NotImplementedError("This script is not meant to be run directly")
