#!/usr/bin/env python

import sys
import argparse
import readline  # Have to import readline to make input() more pleasant for the user (cursor movement etc.)
from pathlib import Path


def init_server(path: Path) -> None:
    """
    Initialize the server: If not yet initialized in path, create empty bare git repository in path/coldbrew.git/
    :param path: The path to initialize coldbrew server in (to place coldbrew.git/ in)
    :return:
    """
    repo_path = path / 'coldbrew.git'

    if repo_path.is_dir():
        print(f"coldbrew server has already been initialized in {repo_path}")
        sys.exit(1)

    resp = input(f"Initialize coldbrew server in {path}? This will create an empty bare git repository in {repo_path}/ [y]/n ")
    if resp == 'y' or resp == '':
        from coldbrew import server
        server.create_bare_repo(repo_path)
        print(f"Initialized coldbrew server in {repo_path}/")
    else:
        print("Aborted.")
        sys.exit(1)


def run_server(path: Path, address: str, port: int) -> None:
    """
    Run the server: If coldbrew.git/ exists, start the flask/waitress web server
    :param path: The path containing results and coldbrew.git/ directory
    :param address: The server address (or hostname) to bind to
    :param port: The server port to bind to
    :return:
    """
    repo_path = path / 'coldbrew.git'

    if not repo_path.is_dir():
        print(f"coldbrew server has not yet been initialized in {path}. Run 'coldbrew server init' to initialize here")
        sys.exit(1)

    from coldbrew import server
    server.run(path, address, port)


def init_client() -> None:
    """
    Initialize the client: If in valid working directory, create coldbrew remote (interactively asking for the url once)
    and coldbrew branch. This will fail if already initialized.
    :return:
    """
    path = Path.cwd()

    from coldbrew import client
    if not client.contains_git_repo(path):
        print(f"No git repository was found in {path}. Run coldbrew init in your working directory")
        sys.exit(1)

    if client.remote_exists(path):
        print(f"coldbrew has already been initialized in {path}")
        sys.exit(1)

    resp = input(f'Initialize coldbrew in {path}? This will add a coldbrew remote and a coldbrew branch to your existing repository [y]/n ')
    if resp == 'y' or resp == '':
        url = input(f'Enter the remote git url (ending with coldbrew.git/): ')
        client.init_coldbrew(path, url)
        print(f"Initialized coldbrew in {path}")
    else:
        print("Aborted.")
        sys.exit(1)


def run_client(description: str) -> None:
    """
    Run the client: Commit and push a snapshot on the coldbrew branch
    Does not perform any initialization; will fail if not initialized yet
    :param description: This snapshot's description; will be the commit message
    :return:
    """
    path = Path.cwd()

    from coldbrew import client

    if not client.contains_git_repo(path):
        print(f"No git repository was found in {path}. Run coldbrew in your working directory")
        sys.exit(1)

    coldbrew_remote_exists = client.remote_exists(path)
    if not coldbrew_remote_exists:
        print(f"coldbrew has not yet been initialized in {path}. Run 'coldbrew init' to initialize here")
        sys.exit(1)

    while not client.remote_valid(path):
        resp = input(f'Cannot reach the configured coldbrew remote. Reconfigure the remote url? [y]/n ')
        if resp == 'y' or resp == '':
            url = input(f'Enter the remote git url (ending with coldbrew.git/): ')
            client.set_remote_url(path, url)
            print(f"Set remote git url to {url}")
        else:
            print("Aborted.")
            sys.exit(1)

    if description == '':
        description = input("Description: ")
    client.run(description, path)


def main() -> None:
    """
    Main entry point for the coldbrew command line utility
    """
    parser = argparse.ArgumentParser(description="冷萃 - Prepares and manages your neural experiments without getting in the way ☕️🥶")
    parser.add_argument('command', nargs='*', type=str, choices=['', 'init', 'server', 'server init'], default='',
                        help="The coldbrew subcommand (analogous to git commands): Run the client by running 'coldbrew' without subcommand; "
                             "initialize the client in the current directory via 'coldbrew init'; "
                             "run the server via coldbrew server; initialize the server in the current directory via 'coldbrew server init'")

    parser.add_argument('-d', '--description', required=False, type=str,
                        help="Used in client mode. This will be the commit message of the created snapshot. The coldbrew server displays this as experiment description")

    parser.add_argument('-a', '--address', required=False, default='localhost', type=str,
                        help="Used in server mode. The address the server should listen on")
    parser.add_argument('-p', '--port', required=False, default=8888, type=int,
                        help="Used in server mode. The port the server should listen on")
    args = parser.parse_args()
    args.command = ' '.join(args.command)

    try:
        # SERVER INIT
        if args.command == 'server init':
            path = Path.cwd()
            init_server(path)

        # SERVER MAIN
        elif args.command == 'server':
            path = Path.cwd()
            run_server(path, args.address, args.port)

        # CLIENT INIT
        elif args.command == 'init':
            init_client()

        # CLIENT MAIN
        elif args.command == '':
            if args.description is None:
                description = ''
            else:
                description = args.description
            run_client(description)

        else:
            print(f"Invalid coldbrew command: {args.command}")
            sys.exit(1)

    except KeyboardInterrupt:
        print("Aborted")
        sys.exit(1)


if __name__ == '__main__':
    raise NotImplementedError("This script is not meant to be run directly")
